@@ -22,6 +22,7 @@
 #include <uuid/uuid.h>
 
 #include "f2fs_fs.h"
+#include "f2fs_format_utils.h"
 
 extern struct f2fs_configuration config;
 struct f2fs_super_block super_block;
@@ -298,7 +299,7 @@ static int f2fs_prepare_super_block(void)
 	 */
 	sit_bitmap_size = ((le32_to_cpu(super_block.segment_count_sit) / 2) <<
 				log_blks_per_seg) / 8;
-	max_nat_bitmap_size = 4096 - sizeof(struct f2fs_checkpoint) + 1 -
+	max_nat_bitmap_size = CHECKSUM_OFFSET - sizeof(struct f2fs_checkpoint) + 1 -
 			sit_bitmap_size;
 	max_nat_segments = (max_nat_bitmap_size * 8) >> log_blks_per_seg;
 
@@ -569,8 +570,7 @@ static int f2fs_write_check_point_pack(void)
 	ckp->cp_pack_start_sum = cpu_to_le32(1);
 	ckp->valid_node_count = cpu_to_le32(1);
 	ckp->valid_inode_count = cpu_to_le32(1);
-	ckp->next_free_nid = cpu_to_le32(
-			le32_to_cpu(super_block.root_ino) + 1);
+	ckp->next_free_nid = cpu_to_le32(0xc00000);
 
 	ckp->sit_ver_bitmap_bytesize = cpu_to_le32(
 			((le32_to_cpu(super_block.segment_count_sit) / 2) <<
@@ -626,7 +626,7 @@ static int f2fs_write_check_point_pack(void)
 	sum->n_sits = cpu_to_le16(6);
 	sum->sit_j.entries[0].segno = ckp->cur_node_segno[0];
 	sum->sit_j.entries[0].se.vblocks = cpu_to_le16((CURSEG_HOT_NODE << 10) | 1);
-	f2fs_set_bit(0, sum->sit_j.entries[0].se.valid_map);
+	f2fs_set_bit(0, (char *)sum->sit_j.entries[0].se.valid_map);
 	sum->sit_j.entries[1].segno = ckp->cur_node_segno[1];
 	sum->sit_j.entries[1].se.vblocks = cpu_to_le16((CURSEG_WARM_NODE << 10));
 	sum->sit_j.entries[2].segno = ckp->cur_node_segno[2];
@@ -635,7 +635,7 @@ static int f2fs_write_check_point_pack(void)
 	/* data sit for root */
 	sum->sit_j.entries[3].segno = ckp->cur_data_segno[0];
 	sum->sit_j.entries[3].se.vblocks = cpu_to_le16((CURSEG_HOT_DATA << 10) | 1);
-	f2fs_set_bit(0, sum->sit_j.entries[3].se.valid_map);
+	f2fs_set_bit(0, (char *)sum->sit_j.entries[3].se.valid_map);
 	sum->sit_j.entries[4].segno = ckp->cur_data_segno[1];
 	sum->sit_j.entries[4].se.vblocks = cpu_to_le16((CURSEG_WARM_DATA << 10));
 	sum->sit_j.entries[5].segno = ckp->cur_data_segno[2];
@@ -702,6 +702,13 @@ static int f2fs_write_check_point_pack(void)
 		return -1;
 	}
 
+	/* 10. cp page 2 of check point pack 2 */
+	cp_seg_blk_offset += blk_size_bytes * (le32_to_cpu(ckp->cp_pack_total_block_count) - 1);
+	if (dev_write(ckp, cp_seg_blk_offset, F2FS_BLKSIZE)) {
+		MSG(1, "\tError: While writing the ckp to disk!!!\n");
+		return -1;
+	}
+
 	free(sum) ;
 	free(ckp) ;
 	return	0;
@@ -788,7 +795,12 @@ static int f2fs_write_root_inode(void)
 
 	memset(raw_node, 0xff, sizeof(struct f2fs_node));
 
-	main_area_node_seg_blk_offset += F2FS_BLKSIZE;
+	/* avoid power-off-recovery based on roll-forward policy */
+	main_area_node_seg_blk_offset = le32_to_cpu(super_block.main_blkaddr);
+	main_area_node_seg_blk_offset += config.cur_seg[CURSEG_WARM_NODE] *
+					config.blks_per_seg;
+        main_area_node_seg_blk_offset *= blk_size_bytes;
+
 	if (dev_write(raw_node, main_area_node_seg_blk_offset, F2FS_BLKSIZE)) {
 		MSG(1, "\tError: While writing the raw_node to disk!!!\n");
 		return -1;
@@ -903,33 +915,6 @@ exit:
 	return err;
 }
 
-int f2fs_trim_device()
-{
-	unsigned long long range[2];
-	struct stat stat_buf;
-
-	if (!config.trim)
-		return 0;
-
-	range[0] = 0;
-	range[1] = config.total_sectors * DEFAULT_SECTOR_SIZE;
-
-	if (fstat(config.fd, &stat_buf) < 0 ) {
-		MSG(1, "\tError: Failed to get the device stat!!!\n");
-		return -1;
-	}
-
-	MSG(0, "Info: Discarding device\n");
-	if (S_ISREG(stat_buf.st_mode))
-		return 0;
-	else if (S_ISBLK(stat_buf.st_mode)) {
-		if (ioctl(config.fd, BLKDISCARD, &range) < 0)
-			MSG(0, "Info: This device doesn't support TRIM\n");
-	} else
-		return -1;
-	return 0;
-}
-
 static int f2fs_format_device(void)
 {
 	int err = 0;
@@ -979,15 +964,7 @@ exit:
 	if (err)
 		MSG(0, "\tError: Could not format the device!!!\n");
 
-	/*
-	 * We should call fsync() to flush out all the dirty pages
-	 * in the block device page cache.
-	 */
-	if (fsync(config.fd) < 0)
-		MSG(0, "\tError: Could not conduct fsync!!!\n");
-
-	if (close(config.fd) < 0)
-		MSG(0, "\tError: Failed to close device file!!!\n");
+	f2fs_finalize_device();
 
 	return err;
 }
